/************************************************************************
 *                              functions.c                             *
 *                                                                      *
 *  2005-05-28 03:01 EDT                                                *
 *  Copyright  2005  Otavio Salvador <otavioo@debian.org>               *
 *                   Luis Mondesi    <lemsx1@gmail.com>                 *
 ************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Standard headers */
#ifndef _GNU_SOURCE
        /* Allow GNU specific funtions: *_unlocked() in stdio.h */
        #define _GNU_SOURCE 
#endif /* _GNU_SOURCE */
#include <errno.h>      /* needed by all file manipulating functions */
#include <stdio.h>      /* NULL and others */
#include <stdlib.h>     /* strtol() exit() */
#include <sys/types.h>  /* needed by regex.h regcomp() */
#include <sys/stat.h>   /* stat() */
#include <unistd.h>     /* fork() getpid() */
#include <string.h>     /* strlen() and str*() fam */
#include <signal.h>     /* kill() */
#include <time.h>       /* nanosleep() */
#include <regex.h>      /* regexec() regcomp() */
#include <glib.h>       /* too many... */

#include "common_macros.h"
#include "xml_parser.h"
#include "functions.h"
#include "video.h"

static gint arg_progress=0;
static pid_t _sub_child=0;
static pid_t _sub_parent=0;
static splashy_video_t video;
static DFBRectangle progressbar;

/**
 * @desc takes care of initilizing all special files needed
 * by splashy
 */
        gboolean
spl_init()
{
        DEBUG_PRINT("Entering spl_init(%s)","");
        const gchar* _spl_fifo_path = spl_get_text(SPL_FIFO_XPATH);
        struct stat _stat;
        /* re-Create the splash FIFO */
        /* FIXME reusing same fifo file if found unlink(_spl_fifo_path); */
	stat(_spl_fifo_path, &_stat);
        if (!S_ISFIFO(_stat.st_mode))
        {
                DEBUG_PRINT("spl_init() attempting to create fifo %s",_spl_fifo_path);
                if ( mkfifo(_spl_fifo_path, 0700) == 0 )
                {
                        return TRUE;
                } else {
                        fprintf(stderr, "Splashy: FIFO creation failed %s\n",_spl_fifo_path);
                        return FALSE; /* assumes that it will fail */
                }
        } 
        fprintf(stderr, "Splashy: reusing FIFO %s\n",_spl_fifo_path);
        return TRUE; /* assumes file exists */
}
        gint
cmd_allow_chvt(void **args)
{
        DEBUG_PRINT("%s\n","cmd_allow_chvt() called");
        video_allow_vt_switching();
        return 0;
}
        gint
cmd_exit(void **args)
{
        /* TODO:
         * getting splashy to switch consoles (tty's) automatically
         * after finishing booting turned out to be a bad idea.
         * I'm living the code here commented out just in case we
         * find a solution for this issue later.
         * For now, we wait until we exit, and then send a chvt
         * from splashy99 when splashy has already exited.
         * Users should customize the tty to which splashy will be
         * switched from /etc/default/splashy (on debian).
         * Nonetheless, we let video_allow_vt_switching() here
         * just don't set vt-switching directfb option there as this 
         * breaks splashy.
         * - Luis Mondesi <lemsx1@gmail.com> 
         */
        DEBUG_PRINT("%s\n","cmd_exit called");
        video_allow_vt_switching();

        /* TODO we should do a nice alpha routine here
         * Fade-to-black comes to mind
         */

        /* free up memory */
        video.ev_buffer->Release (video.ev_buffer); /* input buffer */
        video.keyboard->Release (video.keyboard);
        video.surface->Release (video.surface); /* pix holder */
        video.dfb->Release (video.dfb); /* kill it ! */
        
        /* kill our sub_parent also */
        kill(_sub_parent,15);

        exit(0); /* exit all */
        return 0; /* we never reach this */
}
        gint
cmd_progress(void **args)
{
	if (*(gint*)args[0] < 0 || *(gint*)args[0] > PROGRESS_MAX)
		return -1;

	arg_progress = *(gint *)args[0];

        if ( arg_progress == 0 ) return 1;

        video_update_progressbar(&video, &progressbar, arg_progress);
	return 0;
}

/**
 * @desc our command-holding object and a list of supported commands
 *
 * TODO implement the rest of the commands
 */
struct {
	const gchar *cmd;
	gint (*handler)(void**);
	gint args;
	gchar *specs;
} known_cmds[] = {
/*      {	.cmd = "set theme",
		.handler = cmd_set_theme,
		.args = 1,
		.specs = "s"
	},

	{	.cmd = "set mode",
		.handler = cmd_set_mode,
		.args = 1,
		.specs = "s"
	},

	{	.cmd = "set message",
		.handler = cmd_set_mesg,
		.args = 1,
		.specs = "s"
	},
	
	{	.cmd = "repaint",
		.handler = cmd_repaint,
		.args = 0,
		.specs = NULL,
	},*/
	{	.cmd = "progress",
		.handler = cmd_progress,
		.args = 1,
		.specs = "d"
	},
        {	.cmd = "allowchvt",
		.handler = cmd_allow_chvt,
		.args = 0,
		.specs = NULL,
	},
        {	.cmd = "exit",
		.handler = cmd_exit,
		.args = 0,
		.specs = NULL,
	},
};

/**
 * @desc reads file for pattern for known failure text messages
 * @param perr_pattern a valid regular expression pattern. @see man regex
 * @param filename a valid filename. Can be a special filename like "/dev/vcs1"
 * @return true if error was found
 */
        inline gboolean
spl_search_pattern(const gchar* perr_pattern, const gchar* filename)
{
        regex_t preg;
        gint max_buf = 6200; /* vcs is 6144 chars long. no new-lines are added. man vcs */
        gboolean ret = FALSE; /* assumes we will fail */
        FILE *fp;
        gchar *buffer = g_try_malloc(max_buf);
        if ( buffer && (fp = fopen("/dev/vcs1", "r")) )
        {
                fread_unlocked(buffer, max_buf, 1, fp); /* *_unlocked are a GNU extension: makes function fread() thread safe */
                fclose(fp);
                buffer[max_buf-1] = '\0';
                if ( regcomp(&preg,perr_pattern,
                                        REG_EXTENDED|REG_ICASE|REG_NEWLINE|REG_NOSUB) == 0 )
                {
                        if ( regexec(&preg, buffer, 0, NULL, 0) == 0 )
                                ret = TRUE;
                        regfree(&preg);
                }
        }
        if ( buffer )
                g_free(buffer);
        return ret;
}
/**
 * @desc a simple function that waits for text to be written to a named pipe and dispatches commands accordingly
 * @param _fifo named pipe to read
 * @return
 */
        inline void
spl_read_fifo(FILE* _fifo)
{
        gchar buf[1024]; /** used as buffer for reading FIFO */
        gint i,j,k;

        /* Read FIFO and call functor to handle exceptions */
        while (fgets(buf, 1024, _fifo))
        {
                gchar *t;
                int args_i[4];
                void *args[4];

                buf[1023] = 0;
                buf[strlen(buf)-1] = 0;

                for (i = 0; i < sizeof(known_cmds)/sizeof(known_cmds[0]); i++) {

                        k = strlen(known_cmds[i].cmd);

                        if (strncmp(buf, known_cmds[i].cmd, k))
                                continue;

                        for (j = 0; j < known_cmds[i].args; j++) {

                                for (; buf[k] == ' '; buf[k] = 0, k++);
                                if (!buf[k])
                                        goto out;

                                switch (known_cmds[i].specs[j]) {

                                        case 's':
                                                args[j] = &(buf[k]);
                                                for (; buf[k] != ' '; k++);
                                                break;

                                        case 'd':
                                                args_i[j] = strtol(&(buf[k]), &t, 10);
                                                if (t == &(buf[k]))
                                                        goto out;

                                                args[j] = &(args_i[j]);
                                                k = t - buf;
                                                break;
                                }
                        }
                        /**
                         * cmds should be used for everything
                         * @see known_cmds
                         * and handlers
                         */
                        known_cmds[i].handler(args);
                }
        }
out:	fclose(_fifo);
}

/**
 * @desc _splashy_child handles both start and stop routines for splashy during boot and shutdown respectively
 * Note that the fork() happens before this function is called by its
 * wrappers: spl_child_start() and spl_child_stop()
 * @see spl_child_start() and @see spl_child_stop()
 * @param seq sequence command to execute: "boot" or "shutdown"
 * @param show_progress whether we should display a progress bar
 * @return
 */
        gint
_splashy_child(const gchar* seq, gboolean show_progress)
{
        const gchar* _spl_fifo_path = spl_get_text(SPL_FIFO_XPATH);
        FILE* spl_fifo = NULL;
        const gchar* background = NULL;
        /* 0 seconds and 10 nano seconds: 10 x 1x10^-9 */
        struct timespec _sleep;
        _sleep.tv_sec=0;
        _sleep.tv_nsec=10; 
        if ( g_ascii_strncasecmp(seq,"boot",4) == 0 )
        {
                background = spl_get_text("/splashy/background/boot");
        }  else if ( g_ascii_strncasecmp(seq,"shutdown",8) == 0 )
        {
                background = spl_get_text("/splashy/background/shutdown");
        } else {
                background = spl_get_text("/splashy/background/errorimg");
        }
        DEBUG_PRINT( "_splashy_child() starting splash with image %s",background);
	video_start_splash(&video, background);

        if ( show_progress == TRUE )
        {
                /* starting bars stuff! */
                DEBUG_PRINT( "splashy_child_boot() starting progressbar %s","");
                video_draw_progressbar(&video, &progressbar);
        }

        /* setup an event interface */
        DFBInputEvent event;
        /* setup our main input: keyboard */
        /* TODO would we leak if we don't check if video.keyboard is not NULL ? 
         * We should make sure splashy can't be lanched twice from this same thread */
	DFBCHECK( video.dfb->GetInputDevice(video.dfb, DIDID_KEYBOARD, &video.keyboard) );
        /* create event buffer for the keyboard: to listen for events */
        DFBCHECK( video.keyboard->CreateEventBuffer (video.keyboard, &video.ev_buffer) );

        /* here is where the magic happens, because WaitForEvent*() waits idle
         * for events, we need to fork and handle this in separate threads
         */
        gint child = fork();
        if ( child < 0 )
        {
                fprintf(stderr,"%s\n","Error occurred while creating a child process %s. Bailing out...");
                splashy_child_exit();
        }
        if ( child == 0 )
        {
                /* child: save child pid to be used by sub_parent */
                _sub_child = getpid();
                /* It's important that the progress bar code runs as a child process because
                 * this doesn't need to grab events from the keyboard. All communication
                 * to this is done through a FIFO 
                 */
                /* run our progress bar in a tight loop */
                while (1)
                {
                        /* if no special keys were pressed, read from our
                         * fifo and continue waiting for events
                         */
                        spl_fifo = fopen(_spl_fifo_path, "r");
                        if (!spl_fifo)
                        {
                                if (errno == EINTR)
                                {
                                        /* no events in FIFO yet? release some CPU cycles */
                                        nanosleep(&_sleep,NULL);
                                        continue;
                                }
                                g_print("Can't open the splash FIFO %s for reading",_spl_fifo_path);
                                return -1;
                        }
                        /* fifo is read and a 'handler' takes care of the rest */
                        spl_read_fifo(spl_fifo);
                } /* ends while(1) */
        } else {
                /* save sub_parent pid to be used by sub_child */
                _sub_parent = getpid();
                /* get autoverboseonerror from config file 
                 * and set autoverbose accordingly */
                gboolean autoverbose = FALSE;
                if ( g_ascii_strncasecmp(spl_get_text(SPL_AUTO_VERBOSE_ON_ERROR), "yes", 3) == 0 )
                        autoverbose = TRUE;
                while(1)
                {
                        /* sub-parent (we are a fork after all). init is our parent */
                        DFBCHECK( video.ev_buffer->WaitForEvent(video.ev_buffer) );
                        /* get all events from our event buffer (fifo style) and
                         * process them.
                         */
                        while ( 
                                        video.ev_buffer->HasEvent (video.ev_buffer) == DFB_OK &&
                                        video.ev_buffer->GetEvent (video.ev_buffer, DFB_EVENT(&event)) == DFB_OK 
                              )
                        {
                                /* exit if ESC or F2 is pressed 
                                 * also kill our sub_child while at it */
                                if (event.type == DIET_KEYPRESS)
                                {
                                        if (event.key_id == DIKI_ESCAPE)
                                        {
                                                kill(_sub_child,15);
                                                splashy_child_exit();
                                        } else if (event.key_id == DIKI_F2) {
                                                kill(_sub_child,15); 
                                                splashy_child_exit();
                                        }
                                }
                        } /* ends while ev_buffer */
                        /* now read the /dev/vcs1 console and look for error patterns */
                        if ( autoverbose == TRUE )
                        {
                                /* grouping is needed */
                                if ( spl_search_pattern("(fail|error|abort)","/dev/vcs1") == TRUE )
                                {
                                        /* TODO
                                         * should we switch to splashy errorimg instead of
                                         * completely exiting??
                                         */
                                        kill(_sub_child,15); 
                                        splashy_child_exit();
                                }
                        }
                }
        } /* ends if/else fork child */
        return 0;
}

/**
 * @desc the preferred way to exit splashy:
 */
        void
splashy_child_exit()
{
        cmd_exit(NULL);
}

/**
 * @desc what to do when splashy is starting up (boot process)
 * TODO
 * splashy child start/stop functions should allow argumenst from main.c
 */
        void
splashy_child_start()
{
        /* TODO customize so that we run without progressbar if needed */
        DEBUG_PRINT ("Entering splashy_child_start %s","");
        gint ret = _splashy_child("boot",TRUE);
        if ( ret < 0 )
                fprintf(stderr,"Could not start splashy. %d\n",ret);
        DEBUG_PRINT("Exiting splashy_child_start %d",ret);
}
/**
 * @desc what to do when splashy is in the stopping process (shutdown)
 */
        void
splashy_child_stop()
{
        /* TODO customize so that we run without progressbar if needed */
        gint ret = _splashy_child("shutdown",TRUE);
        if ( ret < 0 )
                g_print("ERROR: main process exited with %d\n",ret);
}
