/************************************************************************
 *                              main.c                                  *
 *                                                                      *
 *  2005-05-28 03:01 EDT                                                *
 *  Copyright  2005   Luis Mondesi    <lemsx1@gmail.com>                *
 *                                                                      *
 ************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <unistd.h>     /* fork() and related */
#include <glib.h>

#include "common_macros.h"
#include "functions.h"
#include "xml_parser.h"

#define USAGE "usage: splashy boot|shutdown"

        int
main(int argc, char *argv[])
{
        DEBUG_PRINT ( "main() invoked %d",argc);
	if ( g_getenv("RUNLEVEL") && 
                        g_ascii_strncasecmp(g_getenv("RUNLEVEL"), 
                                "1", 1) == 0 )
        {
		fprintf(stderr, "%s\n","Single user mode detected. Exiting ...");
		return 1;
	}

        if ( argc < 2 ) {
                g_print("%s\n", USAGE);
                return 1;
        }
        
        if ( ! spl_init() ) /* create FIFO or bail out */
        {
                fprintf(stderr, "%s\n","Error occured while starting splashy."
                "Make sure that you have write access to the directory "
                "holding splashy.fifo");
                /* g_free() memory */
                return 1;
        }
        gint child = fork();
        if ( child < 0 )
        {
                fprintf(stderr, "%s\n","Sorry, could not send splashy to the background. Bailing out...");
                exit(1);
        }
        if ( child == 0 )
        {
                /* child */
                if( g_ascii_strncasecmp( argv[1], "boot", 4 ) == 0 )
                {
                        DEBUG_PRINT("calling splashy_child_start(%s)","");
                        splashy_child_start();
                } else if ( g_ascii_strncasecmp( argv[1], "shutdown", 8 ) == 0 ) {
                        DEBUG_PRINT("calling splashy_child_stop(%s)","");
                        splashy_child_stop();
                } else {
                        DEBUG_PRINT("No argument given %s","");
                        g_print("%s\n", USAGE);
                }
                splashy_child_exit();
        } 
        return 0;
}
