/************************************************************************
 *                              video.c                                 *
 *                                                                      *
 *  2005-05-28 03:01 EDT                                                *
 *  Copyright  2005  Otavio Salvador <otavio@debian.org>                *
 *                   Luis Mondesi    <lemsx1@gmail.com>                 *
 ************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "common_macros.h"
#include "xml_parser.h"
#include "video.h"

static gint _last_progress = 0;
static const gchar* _current_background = NULL;

/* Public members */
        void
video_allow_vt_switching()
{
        DEBUG_PRINT ("video_allow_vt_switching(%s) called","");
        /* Please read functions.c about why NOT to set
         * vt-switching here. 
         * 2005-04-28 20:42 EDT - Luis Mondesi <lemsx1@gmail.com> */
        /* DirectFBSetOption( "vt-switching", NULL);  */
        DirectFBSetOption( "force-windowed", NULL); 
}
        gint
video_draw_progressbar(splashy_video_t *video, 
                DFBRectangle *progressbar)
{
        /* Store screen SIZE */
	gint screenWidth, screenHeight;

        /* Retangle colors */
	gint retangle_red, retangle_green, retangle_blue, retangle_alpha;

        /* retrieve information */
	video->surface->GetSize(video->surface, &screenWidth, &screenHeight);
	progressbar->x = screenWidth*
                spl_get_int("/splashy/progressbar/x",10)/100;
	progressbar->y = screenHeight*
                spl_get_int("/splashy/progressbar/y",10)/100;
	progressbar->w = screenWidth*
                spl_get_int("/splashy/progressbar/width",10)/100;
	progressbar->h = screenHeight*
                spl_get_int("/splashy/progressbar/height",10)/100;

        if (    progressbar->x < 0 || 
                progressbar->y < 0 ||
                progressbar->w < 0 ||
                progressbar->h < 0 )
        {
                g_print("ERROR: progress bar coordinates (x,y,width,height) had negative values. Please check your configuration exists and it's readable\n");
                return 1;
        }

	video->surface->DrawRectangle(video->surface,
			       progressbar->x,
			       progressbar->y,
			       progressbar->w,
			       progressbar->h);

	retangle_red = spl_get_int("/splashy/progressbar/red",10);
	retangle_green = spl_get_int("/splashy/progressbar/green",10);
	retangle_blue = spl_get_int("/splashy/progressbar/blue",10);
	retangle_alpha = spl_get_int("/splashy/progressbar/alpha",10);

        /*set the filling color of the progressbar*/
	if(retangle_red > 255 ||
	   retangle_green > 255 ||
	   retangle_blue > 255 ||
	   retangle_alpha > 255) {
		g_print("ERROR: red, green, blue or alpha tags in the configuration file contain wrong values");
		return 3;
	}

	video->surface->SetColor(video->surface,
			  retangle_red,
			  retangle_green,
			  retangle_blue,
			  retangle_alpha);
	return 0;
}

        gint
video_update_progressbar(splashy_video_t *video, 
                DFBRectangle *progressbar, gint perc)
{
        DEBUG_PRINT ("video_update_progressbar(%d)",perc);
        if ( perc < 0 ) return 1; /* sanity check */
        if ( video == NULL ) return 1;
        if ( perc < _last_progress )
        {
                /* progress in reverse */
                /* FIXME this might be too slow... */
                video_change_splash(video, _current_background);
                video_draw_progressbar(video, progressbar);
        }
        /* draw progress */
        video->surface->FillRectangle(video->surface,
                        progressbar->x,
                        progressbar->y,
                        progressbar->w*perc/100,
                        progressbar->h);
        /* save last perc value */
        _last_progress = perc;
	return 0;
}

        void
video_set_mode(splashy_video_t *video)
{
        /* dfb->EnumVideoModes (dfb, EnumModesCallback, this); */
        /* used to disable inputs from mouse and reduce overhead
	   (about 15% of CPU usage)*/
	video->dfb->SetCooperativeLevel(video->dfb, DFSCL_FULLSCREEN);
}

        void
video_start_splash(splashy_video_t *video, const gchar* background)
{
        /*initializing Directfb*/
        /* 
         * - no-debug           := suppresses debug messages
         * - quiet              := suppresses all messages (but debug)
         * - graphics-vt        := puts directfb vt in graphics mode
         * - no-cursor          := disallow showing a cursor
         */
       
        _current_background = background;

        DFBCHECK( DirectFBInit(NULL, NULL) );
        DirectFBSetOption( "quiet", NULL );
        DirectFBSetOption( "no-debug", NULL );
        DirectFBSetOption( "graphics-vt", NULL );
        DirectFBSetOption( "no-cursor", NULL );

	DFBCHECK( DirectFBCreate(&video->dfb) );

	video_set_mode(video);

	DFBCHECK( video->dfb->CreateImageProvider(video->dfb, background, 
                        &video->provider) );
	DFBCHECK( video->provider->GetSurfaceDescription( video->provider, 
                        &video->desc ) );
        /* TODO use CreateInputEventBuffer with caps to allow
         * the fullscreen surface to grab keyboard events.
         * this need to be store in the video struct
         */
        /*flags to set the default surface as main surface*/
        video->desc.flags = DSDESC_CAPS;
	video->desc.caps = DSCAPS_PRIMARY;

	DFBCHECK( video->dfb->CreateSurface( video->dfb, &video->desc, 
                        &video->surface) );
        
        /* allow surface to have alpha channels */
        DFBCHECK( video->surface->SetBlittingFlags (video->surface, DSBLIT_BLEND_ALPHACHANNEL) );

        /*it writes on the framebuffer the background image*/
	video->provider->RenderTo( video->provider, video->surface, 
                        NULL );
}

        void
video_change_splash(splashy_video_t *video, const gchar *newimage)
{
	video->dfb->CreateImageProvider(video->dfb, newimage, 
                        &video->provider);
	video->provider->RenderTo( video->provider, video->surface, 
                        NULL );
}
